<?php

namespace Fastrack\Utilities\ServicesApi;

use DateTime;

class BookRequest implements Request
{
	private DateTime $arrive;
	private DateTime $depart;
	private int $propertyId;
	private int $roomId;
	private int $rateId;
	private string $promo;
	private string $channel;
	private string $bookingSource;
	private string $travelAgent;
	private int $adults;
	private int $children;
	private int $infants;
	private Payment $payment;
	private Customer $customer;
	/** @var array<TransactionItem> */
	private array $transactionItems = [];

	public function buildRequest(): array
	{
		$DATE_FORMAT = 'Y-m-d';
		$transactionItems = [];
		foreach ($this->transactionItems as $transactionItem) {
			$transactionItems[] = $transactionItem->buildRequest();
		}
		return [
			'arrive' => $this->arrive->format($DATE_FORMAT),
			'depart' => $this->depart->format($DATE_FORMAT),
			'propertyId' => $this->propertyId,
			'roomId' => $this->roomId,
			'rateId' => $this->rateId,
			'promo' => $this->promo,
			'channel' => $this->channel,
			'bookingSource' => $this->bookingSource,
			'travelAgent' => $this->travelAgent,
			'adults' => $this->adults,
			'children' => $this->children,
			'infants' => $this->infants,
			'payment' => $this->payment->buildRequest(),
			'customer' => $this->customer->buildRequest(),
			'transactionItems' => $transactionItems,
		];
	}

	/**
	 * @param DateTime $arrive
	 */
	public function setArrive(DateTime $arrive): void
	{
		$this->arrive = $arrive;
	}

	/**
	 * @param DateTime $depart
	 */
	public function setDepart(DateTime $depart): void
	{
		$this->depart = $depart;
	}

	/**
	 * @param int $propertyId
	 */
	public function setPropertyId(int $propertyId): void
	{
		$this->propertyId = $propertyId;
	}

	/**
	 * @return int
	 */
	public function getPropertyId(): int
	{
		return $this->propertyId;
	}

	/**
	 * @param int $roomId
	 */
	public function setRoomId(int $roomId): void
	{
		$this->roomId = $roomId;
	}

	/**
	 * @param int $rateId
	 */
	public function setRateId(int $rateId): void
	{
		$this->rateId = $rateId;
	}

	/**
	 * @param string $promo
	 */
	public function setPromo(string $promo): void
	{
		$this->promo = $promo;
	}

	/**
	 * @param string $channel
	 */
	public function setChannel(string $channel): void
	{
		$this->channel = $channel;
	}

	/**
	 * @param string $bookingSource
	 */
	public function setBookingSource(string $bookingSource): void
	{
		$this->bookingSource = $bookingSource;
	}

	/**
	 * @param string $travelAgent
	 */
	public function setTravelAgent(string $travelAgent): void
	{
		$this->travelAgent = $travelAgent;
	}

	/**
	 * @param int $adults
	 */
	public function setAdults(int $adults): void
	{
		$this->adults = $adults;
	}

	/**
	 * @param int $children
	 */
	public function setChildren(int $children): void
	{
		$this->children = $children;
	}

	/**
	 * @param int $infants
	 */
	public function setInfants(int $infants): void
	{
		$this->infants = $infants;
	}

	/**
	 * @param Payment $payment
	 */
	public function setPayment(Payment $payment): void
	{
		$this->payment = $payment;
	}

	/**
	 * @param Customer $customer
	 */
	public function setCustomer(Customer $customer): void
	{
		$this->customer = $customer;
	}

	/**
	 * @param array<TransactionItem> $transactionItems
	 */
	public function setTransactionItems(array $transactionItems): void
	{
		$this->transactionItems = $transactionItems;
	}

}
